/*
 * Copyright (c) 2015, 2017, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package com.oracle.svm.core.log;

import java.io.FileDescriptor;
import java.io.FileOutputStream;

import com.oracle.svm.core.SubstrateUtil;
import com.oracle.svm.core.annotate.Uninterruptible;

public class FileLog extends RealLog {
    private FileOutputStream out;

    public FileLog setOutputStream(FileOutputStream out) {
        this.out = out;
        return this;
    }

    @Uninterruptible(reason = "Called from uninterruptible code.")
    @Override
    protected FileDescriptor getOutputFile() {
        if (out == null) {
            /*
             * Avoid a NullPointerException when log output file is not yet set, e.g., during early
             * startup phases.
             */
            return FileDescriptor.err;
        }
        return SubstrateUtil.getFileDescriptor(out);
    }
}
