/*
 * Copyright (c) 2017, 2017, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package com.oracle.svm.junit;

import com.oracle.svm.core.annotate.Alias;
import com.oracle.svm.core.annotate.RecomputeFieldValue;
import com.oracle.svm.core.annotate.RecomputeFieldValue.CustomFieldValueComputer;
import com.oracle.svm.core.annotate.RecomputeFieldValue.Kind;
import com.oracle.svm.core.annotate.Substitute;
import com.oracle.svm.core.annotate.TargetClass;
import jdk.vm.ci.meta.ResolvedJavaField;
import org.junit.runner.Description;

@TargetClass(value = Description.class, onlyWith = JUnitFeature.IsEnabled.class)
public final class Target_org_junit_runner_Description {

    public static final class TestClassComputer implements CustomFieldValueComputer {

        @Override
        public Object compute(ResolvedJavaField original, ResolvedJavaField annotated, Object receiver) {
            Description description = (Description) receiver;
            return description.getTestClass();
        }
    }

    @Alias @RecomputeFieldValue(kind = Kind.Custom, declClass = TestClassComputer.class) Class<?> fTestClass;

    @Substitute
    public Class<?> getTestClass() {
        return fTestClass;
    }
}
